%-------------------------------------------------------------------------------
%
% Filename:  csxdgemm.m
%
% Description:  MATLAB script that multiplies two identical, square matrices
% of varying size and plots the resulting GF/s performance on the screen.
%
% Copyright (c) 2006 ClearSpeed Technology plc. All rights reserved.
% 
% This material is provided "as is", with absolutely no warranty expressed
% or implied. Any use is at your own risk. Permission to use or copy this
% software for any purpose is hereby granted without fee, provided the
% above notices are retained on all copies. Permission to modify the code
% and to distribute modified code is granted, provided the above notices
% are retained, and a notice that the code was modified is included with
% the above copyright notice. 
%
%-------------------------------------------------------------------------------

% Set the min, max, and step sizes for the matrices
% -------------------------------------------------
lowerLimit=64;  %32, 64, 96, 192
upperLimit=5760; %192*30=5760, 192*32=6144
stepSize=96;    %32, 64, 96, 192

% Initialize plotting
% -------------------
figure;
hold on
axis ([lowerLimit upperLimit 0 40]);
%title('ClearSpeed CSX600 DGEMM Performance in MATLAB');
title('Example Plot of DGEMM Results');
xlabel('Matrix size');
ylabel('GFLOPS');
grid on;

% Open an output file for recording performance rates
% ---------------------------------------------------
%fid=fopen('speed_cs.txt','w');

% Initialize variables
% --------------------
index=1;
billion=1000000000;
clear a;
clear gflops;

% Loop over matrices and plot the speed of the matrix multiply operation
% ----------------------------------------------------------------------
for m=lowerLimit:stepSize:upperLimit
    a=rand(m);					% generate a random, m-by-m matrix
    tic;					% start the timer
    a*a;					% multiply matrices
    t=toc;					% stop the timer
    gflops(index)=((2*m*m*m)/billion)/t;	% compute GFLOP/s

    plot(lowerLimit:stepSize:m, gflops);	% update the performance plot
    legend('Dual CSX600 DGEMM', 2);
    drawnow

    %fprintf(fid, '%d %12.4f \n', m, gflops(index)); % append to the output file
    index=index+1;				% get ready for next pass
end

% Close the output file
% ---------------------
%fclose(fid);
